@@ -4,123 +4,147 @@ no strict 'refs';  # Let's just make this very clear right off
 
 use Carp;
 use vars qw($VERSION);
-$VERSION = do { my @r = (q$Revision: 1.11 $ =~ /\d+/g); sprintf '%d.%03d'.'%02d' x ($#r-1), @r};
+$VERSION = do { my @r = (q$Revision: 1.13 $ =~ /\d+/g); sprintf '%d.%03d'.'%02d' x ($#r-1), @r};
 
 my $Ident = '[^\W_0-9]\w*';
 
-use constant ENUM		=> 1;
-use constant BITMASK	=> 2;
+sub ENUM    () { 1 }
+sub BITMASK () { 2 }
 
 sub import {
-	my $class	= shift;
-	@_ or return;		# Ignore 'use enum;'
-	my $pkg		= caller() . '::';
-	my $prefix	= '';	# default no prefix 
-	my $index	= 0;	# default start index
-	my $mode	= ENUM;	# default to enum
+    my $class   = shift;
+    @_ or return;       # Ignore 'use enum;'
+    my $pkg     = caller() . '::';
+    my $prefix  = '';   # default no prefix 
+    my $index   = 0;    # default start index
+    my $mode    = ENUM; # default to enum
 
-	## Pragmas should be as fast as they can be, so we inline some
-	## pieces.
-	foreach (@_) {
-		## Plain tag is most common case
-		if (/^$Ident$/o) {
-			my $n = $index;
+    ## Pragmas should be as fast as they can be, so we inline some
+    ## pieces.
+    foreach (@_) {
+        ## Plain tag is most common case
+        if (/^$Ident$/o) {
+            my $n = $index;
 
-			if ($mode == ENUM) {
-				$index++;
-			}
-			elsif ($mode == BITMASK) {
-				$index ||= 1;
-				$index *= 2;
-				if ( $index & ($index - 1) ) {
-					croak (
-						"$index is not a valid single bitmask "
-						. " (Maybe you overflowed your system's max int value?)"
-					);
-				}
-			}
-			else {
-				confess qq(Can't Happen: mode $mode invalid);
-			}
+            if ($mode == ENUM) {
+                $index++;
+            }
+            elsif ($mode == BITMASK) {
+                $index ||= 1;
+                $index *= 2;
+                if ( $index & ($index - 1) ) {
+                    croak (
+                        "$index is not a valid single bitmask "
+                        . " (Maybe you overflowed your system's max int value?)"
+                    );
+                }
+            }
+            else {
+                confess qq(Can't Happen: mode $mode invalid);
+            }
 
-			*{"$pkg$prefix$_"} = sub () { $n };
-		}
+            *{"$pkg$prefix$_"} = sub () { $n };
+        }
 
-		## Index change
-		elsif (/^($Ident)=(.+)$/o) {
-			$index	= $2;
-			my $n	= $index;
+        ## Index change
+        elsif (/^($Ident)=(.+)$/o) {
+            my $name= $1;
+            $index  = $2;
 
-			if ($mode == BITMASK) {
-				($index & ($index - 1))
-					and croak "$index is not a valid single bitmask";
-				$index *= 2;
-			}
-			elsif ($mode == ENUM) {
-				$index++;
-			}
-			else {
-				confess qq(Can't Happen: mode $mode invalid);
-			}
+            ## Convert non-decimal numerics to decimal
+            if ($index =~ /^0x\d/i) {       ## Hex
+                $index = hex $index;
+            }
+            elsif ($index =~ /^0\d/) {      ## Oct
+                $index = oct $index;
+            }
+            elsif ($index !~ /[^\d_]/) {    ## 123_456 notation
+                $index =~ s/_//;
+            }
 
-			*{"$pkg$prefix$1"} = sub () { $n };
-		}
+            my $n = $index;
 
-		## Prefix/option change
-		elsif (/^([A-Z]*):($Ident)?(=?)(.*)/) {
-			## Option change
-			if ($1) {
-				if		($1 eq 'ENUM')		{ $mode = ENUM;		$index = 0 }
-				elsif	($1 eq 'BITMASK')	{ $mode = BITMASK;	$index = 1 }
-				else	{ croak qq(Invalid enum option '$1') }
-			}
+            if ($mode == BITMASK) {
+                ($index & ($index - 1))
+                    and croak "$index is not a valid single bitmask";
+                $index *= 2;
+            }
+            elsif ($mode == ENUM) {
+                $index++;
+            }
+            else {
+                confess qq(Can't Happen: mode $mode invalid);
+            }
 
-			## Index change too?
-			if ($3) {
-				if (length $4) {
-					$index = $4;
+            *{"$pkg$prefix$name"} = sub () { $n };
+        }
 
-					## Bitmask mode must check index changes
-					if ($mode == BITMASK) {
-						($index & ($index - 1))
-							and croak "$index is not a valid single bitmask";
-					}
-				}
-				else {
-					croak qq(No index value defined after "=");
-				}
-			}
+        ## Prefix/option change
+        elsif (/^([A-Z]*):($Ident)?(=?)(.*)/) {
+            ## Option change
+            if ($1) {
+                if      ($1 eq 'ENUM')      { $mode = ENUM;     $index = 0 }
+                elsif   ($1 eq 'BITMASK')   { $mode = BITMASK;  $index = 1 }
+                else    { croak qq(Invalid enum option '$1') }
+            }
 
-			## Incase it's a null prefix
-			$prefix = defined $2 ? $2 : '';
-		}
+            ## Index change too?
+            if ($3) {
+                if (length $4) {
+                    $index = $4;
 
-		## A..Z case magic lists
-		elsif (/^($Ident)\.\.($Ident)$/o) {
-			## Almost never used, so check last
-			foreach my $name ("$1" .. "$2") {
-				my $n = $index;
+                    ## Convert non-decimal numerics to decimal
+                    if ($index =~ /^0x\d/i) {       ## Hex
+                        $index = hex $index;
+                    }
+                    elsif ($index =~ /^0\d/) {      ## Oct
+                        $index = oct $index;
+                    }
+                    elsif ($index !~ /[^\d_]/) {    ## 123_456 notation
+                        $index =~ s/_//;
+                    }
 
-				if ($mode == BITMASK) {
-					($index & ($index - 1))
-						and croak "$index is not a valid single bitmask";
-					$index *= 2;
-				}
-				elsif ($mode == ENUM) {
-					$index++;
-				}
-				else {
-					confess qq(Can't Happen: mode $mode invalid);
-				}
+                    ## Bitmask mode must check index changes
+                    if ($mode == BITMASK) {
+                        ($index & ($index - 1))
+                            and croak "$index is not a valid single bitmask";
+                    }
+                }
+                else {
+                    croak qq(No index value defined after "=");
+                }
+            }
 
-				*{"$pkg$prefix$name"} = sub () { $n };
-			}
-		}
+            ## Incase it's a null prefix
+            $prefix = defined $2 ? $2 : '';
+        }
 
-		else {
-			croak qq(Can't define "$_" as enum type (name contains invalid characters));
-		}
-	}
+        ## A..Z case magic lists
+        elsif (/^($Ident)\.\.($Ident)$/o) {
+            ## Almost never used, so check last
+            foreach my $name ("$1" .. "$2") {
+                my $n = $index;
+
+                if ($mode == BITMASK) {
+                    ($index & ($index - 1))
+                        and croak "$index is not a valid single bitmask";
+                    $index *= 2;
+                }
+                elsif ($mode == ENUM) {
+                    $index++;
+                }
+                else {
+                    confess qq(Can't Happen: mode $mode invalid);
+                }
+
+                *{"$pkg$prefix$name"} = sub () { $n };
+            }
+        }
+
+        else {
+            croak qq(Can't define "$_" as enum type (name contains invalid characters));
+        }
+    }
 }
 
 1;
@@ -205,7 +229,7 @@ a single integer.
     }
     if ($foo & (MY_BAR | MY_DOG)) {
         print "foo has either the MY_BAR or MY_DOG option set\n"
-	}
+    }
 
     $foo ^= MY_DOG;  ## Turn MY_DOG option off (set its bit to false)
 
@@ -252,36 +276,50 @@ ever use this feature...?
 =head1 HISTORY
 
   $Log: enum.pm,v $
+  Revision 1.13  1999/05/13 10:52:30  byron
+
+
+  Fixed auto-index bugs in new non-decimal numeric support.
+
+  Revision 1.12  1999/05/13 10:00:45  byron
+
+
+  Added support for non-decimal numeric representations ala 0x123, 0644, and
+  123_456.
+
+  First version committed to CVS.
+
+
   Revision 1.11  1998/07/18 17:53:05  byron
-  	-Added BITMASK and ENUM directives.
-  	-Revamped documentation.
+    -Added BITMASK and ENUM directives.
+    -Revamped documentation.
 
   Revision 1.10  1998/06/12 20:12:50  byron
-  	-Removed test code
-  	-Released to CPAN
+    -Removed test code
+    -Released to CPAN
 
   Revision 1.9  1998/06/12 00:21:00  byron
- 	-Fixed -w warning when a null tag is used
+    -Fixed -w warning when a null tag is used
 
   Revision 1.8  1998/06/11 23:04:53  byron
-	-Fixed documentation bugs
-	-Moved A..Z case to last as it's not going to be used
-	 as much as the other cases.
+    -Fixed documentation bugs
+    -Moved A..Z case to last as it's not going to be used
+     as much as the other cases.
 
   Revision 1.7  1998/06/10 12:25:04  byron
-	-Changed interface to match original design by Tom Phoenix
-	 as implemented in an early version of enum.pm by Benjamin Holzman.
-	-Changed tag syntax to not require the 'PREFIX' string of Tom's
-	 interface.
-	-Allow multiple prefix tags to be used at any point.
-	-Allowed index value changes from tags.
+    -Changed interface to match original design by Tom Phoenix
+     as implemented in an early version of enum.pm by Benjamin Holzman.
+    -Changed tag syntax to not require the 'PREFIX' string of Tom's
+     interface.
+    -Allow multiple prefix tags to be used at any point.
+    -Allowed index value changes from tags.
 
   Revision 1.6  1998/06/10 03:37:57  byron
-	-Fixed superfulous -w warning
+    -Fixed superfulous -w warning
 
   Revision 1.4  1998/06/10 01:07:03  byron
-	-Changed behaver to closer resemble C enum types
-	-Changed docs to match new behaver
+    -Changed behaver to closer resemble C enum types
+    -Changed docs to match new behaver
 
 =head1 AUTHOR
 
